# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'Speech.section'
        db.add_column('speeches_speech', 'section',
                      self.gf('django.db.models.fields.related.ForeignKey')(to=orm['speeches.Section'], null=True, blank=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'Speech.section'
        db.delete_column('speeches_speech', 'section_id')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'instances.instance': {
            'Meta': {'object_name': 'Instance'},
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'created_instances'", 'null': 'True', 'to': "orm['auth.User']"}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('instances.fields.DNSLabelField', [], {'unique': 'True', 'max_length': '63', 'db_index': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'users': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'instances'", 'blank': 'True', 'to': "orm['auth.User']"})
        },
        'speeches.debate': {
            'Meta': {'object_name': 'Debate'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['instances.Instance']"}),
            'meeting': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['speeches.Meeting']", 'null': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'speeches.meeting': {
            'Meta': {'object_name': 'Meeting'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['instances.Instance']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'speeches.recording': {
            'Meta': {'object_name': 'Recording'},
            'audio': ('django.db.models.fields.files.FileField', [], {'max_length': '255'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['instances.Instance']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'timestamps': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': "orm['speeches.RecordingTimestamp']", 'null': 'True', 'blank': 'True'})
        },
        'speeches.recordingtimestamp': {
            'Meta': {'object_name': 'RecordingTimestamp'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['instances.Instance']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'speaker': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['speeches.Speaker']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'timestamp': ('django.db.models.fields.DateTimeField', [], {'db_index': 'True'})
        },
        'speeches.section': {
            'Meta': {'object_name': 'Section'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'children'", 'null': 'True', 'to': "orm['speeches.Section']"}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'})
        },
        'speeches.speaker': {
            'Meta': {'object_name': 'Speaker'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['instances.Instance']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.TextField', [], {'db_index': 'True'}),
            'popit_url': ('django.db.models.fields.TextField', [], {'blank': 'True'})
        },
        'speeches.speech': {
            'Meta': {'object_name': 'Speech'},
            'audio': ('django.db.models.fields.files.FileField', [], {'max_length': '255', 'blank': 'True'}),
            'celery_task_id': ('django.db.models.fields.CharField', [], {'max_length': '256', 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'debate': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['speeches.Debate']", 'null': 'True', 'blank': 'True'}),
            'end_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'end_time': ('django.db.models.fields.TimeField', [], {'null': 'True', 'blank': 'True'}),
            'event': ('django.db.models.fields.TextField', [], {'db_index': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['instances.Instance']"}),
            'location': ('django.db.models.fields.TextField', [], {'db_index': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'section': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['speeches.Section']", 'null': 'True', 'blank': 'True'}),
            'source_url': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'speaker': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['speeches.Speaker']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'start_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'start_time': ('django.db.models.fields.TimeField', [], {'null': 'True', 'blank': 'True'}),
            'tags': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': "orm['speeches.Tag']", 'null': 'True', 'blank': 'True'}),
            'text': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'title': ('django.db.models.fields.TextField', [], {'blank': 'True'})
        },
        'speeches.tag': {
            'Meta': {'object_name': 'Tag'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'instance': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['instances.Instance']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'})
        }
    }

    complete_apps = ['speeches']



from setuptools import setup, find_packages
import os
import sys

file_dir = os.path.abspath(os.path.dirname(__file__))

def read_file(filename):
    filepath = os.path.join(file_dir, filename)
    return open(filepath).read()

# Fix for Select2 py3 branching
if sys.version_info >= (3,):
    select2 = 'Django-Select2-Py3 >= 4.2.1, < 4.3'
    dateutil = 'python-dateutil >= 2'
else:
    select2 = 'Django-Select2 >= 4.2.2, < 4.3'
    dateutil = 'python-dateutil < 2'

setup(
    name="django-sayit",
    version='1.2.1',
    description='A data store for speeches and transcripts to make them searchable and pretty.',
    long_description=read_file('README.rst'),
    author='mySociety',
    author_email='sayit@mysociety.org',
    url='https://github.com/mysociety/sayit',
    packages=find_packages(exclude=('example_project', 'example_project.*')),
    include_package_data=True,
    install_requires=[
        'psycopg2 >= 2.5.1, < 2.6',
        'pytz >= 2013d',
        'six >= 1.4.1',
        'Django >= 1.4.2, < 1.8',
        select2,
        'django-qmethod == 0.0.3',
        'audioread >= 1.0.1',
        'pyelasticsearch >= 0.6, < 0.7',
        'django-haystack >= 2.1, < 2.2',
        'django-bleach >= 0.2.1',
        'mysociety-django-popolo >= 0.0.2',
        'mysociety-django-sluggable >= 0.2.6',
        'django-subdomain-instances >= 0.6',
        'easy-thumbnails >= 2.1',
        'unicode-slugify == 0.1.1',
    ],
    extras_require={
        'test': [
            'selenium',
            'mock',
            'django-nose == 1.2',
            'Mutagen',
            'lxml',
            dateutil,
        ],
        'develop': [
            'django-debug-toolbar',
            'South == 1.0',
            'popit-django == 0.0.3',
        ],
        'scraping': [
            'beautifulsoup4',
            'requests_cache',
        ],
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Topic :: Database :: Front-Ends',
        'Topic :: Internet :: WWW/HTTP',
    ],
)



"""Copy of the paginate function from django-pagination.
The rest of django-pagination isn't used, and this
lets us fix the Python3 issue too. BSD."""

try:
    set
except NameError:
    from sets import Set as set

from django import template
from django.conf import settings

register = template.Library()

DEFAULT_WINDOW = getattr(settings, 'PAGINATION_DEFAULT_WINDOW', 4)

def paginate(context, window=DEFAULT_WINDOW, hashtag=''):
    """
    Renders the ``pagination/pagination.html`` template, resulting in a
    Digg-like display of the available pages, given the current page.  If there
    are too many pages to be displayed before and after the current page, then
    elipses will be used to indicate the undisplayed gap between page numbers.

    Requires one argument, ``context``, which should be a dictionary-like data
    structure and must contain the following keys:

    ``paginator``
        A ``Paginator`` or ``QuerySetPaginator`` object.

    ``page_obj``
        This should be the result of calling the page method on the
        aforementioned ``Paginator`` or ``QuerySetPaginator`` object, given
        the current page.

    This same ``context`` dictionary-like data structure may also include:

    ``getvars``
        A dictionary of all of the **GET** parameters in the current request.
        This is useful to maintain certain types of state, even when requesting
        a different page.
        """
    try:
        paginator = context['paginator']
        page_obj = context['page_obj']
        page_range = paginator.page_range
        # Calculate the record range in the current page for display.
        records = {'first': 1 + (page_obj.number - 1) * paginator.per_page}
        records['last'] = records['first'] + paginator.per_page - 1
        if records['last'] + paginator.orphans >= paginator.count:
            records['last'] = paginator.count
        # First and last are simply the first *n* pages and the last *n* pages,
        # where *n* is the current window size.
        first = set(page_range[:window])
        last = set(page_range[-window:])
        # Now we look around our current page, making sure that we don't wrap
        # around.
        current_start = page_obj.number-1-window
        if current_start < 0:
            current_start = 0
        current_end = page_obj.number-1+window
        if current_end < 0:
            current_end = 0
        current = set(page_range[current_start:current_end+1])
        pages = []
        # If there's no overlap between the first set of pages and the current
        # set of pages, then there's a possible need for elusion.
        if len(first.intersection(current)) == 0:
            first_list = list(first)
            first_list.sort()
            second_list = list(current)
            second_list.sort()
            pages.extend(first_list)
            diff = second_list[0] - first_list[-1]
            # If there is a gap of two, between the last page of the first
            # set and the first page of the current set, then we're missing a
            # page.
            if diff == 2:
                pages.append(second_list[0] - 1)
            # If the difference is just one, then there's nothing to be done,
            # as the pages need no elusion and are correct.
            elif diff == 1:
                pass
            # Otherwise, there's a bigger gap which needs to be signaled for
            # elusion, by pushing a None value to the page list.
            else:
                pages.append(None)
            pages.extend(second_list)
        else:
            unioned = list(first.union(current))
            unioned.sort()
            pages.extend(unioned)
        # If there's no overlap between the current set of pages and the last
        # set of pages, then there's a possible need for elusion.
        if len(current.intersection(last)) == 0:
            second_list = list(last)
            second_list.sort()
            diff = second_list[0] - pages[-1]
            # If there is a gap of two, between the last page of the current
            # set and the first page of the last set, then we're missing a
            # page.
            if diff == 2:
                pages.append(second_list[0] - 1)
            # If the difference is just one, then there's nothing to be done,
            # as the pages need no elusion and are correct.
            elif diff == 1:
                pass
            # Otherwise, there's a bigger gap which needs to be signaled for
            # elusion, by pushing a None value to the page list.
            else:
                pages.append(None)
            pages.extend(second_list)
        else:
            differenced = list(last.difference(current))
            differenced.sort()
            pages.extend(differenced)
        to_return = {
            'MEDIA_URL': settings.MEDIA_URL,
            'pages': pages,
            'records': records,
            'page_obj': page_obj,
            'paginator': paginator,
            'hashtag': hashtag,
            'is_paginated': paginator.count > paginator.per_page,
        }
        if 'request' in context:
            getvars = context['request'].GET.copy()
            if 'page' in getvars:
                del getvars['page']
            if len(getvars.keys()) > 0:
                to_return['getvars'] = "&%s" % getvars.urlencode()
            else:
                to_return['getvars'] = ''
        return to_return
    except (KeyError, AttributeError):
        return {}

register.inclusion_tag('pagination/pagination.html', takes_context=True)(
    paginate)



from speeches.management.import_commands import ImportCommand
from speeches.importers.import_akomantoso import ImportAkomaNtoso

class Command(ImportCommand):
    importer_class = ImportAkomaNtoso
    document_extension = 'xml'



import os

# Path to here is something like
# /data/vhost/<vhost>/<repo>/<project_name>/settings/base.py
SETTINGS_DIR = os.path.abspath(os.path.dirname(__file__))
PROJECT_DIR = os.path.abspath(os.path.join(SETTINGS_DIR, '..'))
PROJECT_ROOT = os.path.abspath(os.path.join(PROJECT_DIR, '..'))
PARENT_DIR = os.path.abspath(os.path.join(PROJECT_ROOT, '..'))




digits = "0123456789abcdefghjkmnpqrstvwxyz"

class MistypedIDException(Exception):
    pass

def int_to_base32(i):
    """Converts an integer to a base32 string"""
    enc = ''
    while i>=32:
        i, mod = divmod(i,32)
        enc = digits[mod] + enc
    enc = digits[i] + enc
    return enc

def base32_to_int(s):
    """Convert a base 32 string to an integer"""
    mistyped = False
    if s.find('o')>-1 or s.find('i')>-1 or s.find('l')>-1:
        s = s.replace('o', '0').replace('i', '1').replace('l', '1')
        mistyped = True
    decoded = 0
    multi = 1
    while len(s) > 0:
        decoded += multi * digits.index(s[-1:])
        multi = multi * 32
        s = s[:-1]
    if mistyped:
        raise MistypedIDException(decoded)
    return decoded





""""Some helper classes for writing scrapers.

These have been used to write scrapers from several sources for
sayit.mysociety.org including The Leveson Inquiry and the trial of Charles
Taylor.

WARNING: They're provided here in case they are useful to anyone writing
scrapers of their own, but should definitely not be considered part of a stable
API.

You'll need to install some extra dependencies if you want to use this module.
They're listed in requirements-scraping.txt and can be installed like this:

pip install -r requirements-scraping.txt
"""

import itertools
import os
import socket
import re

from exceptions import NotImplementedError, StandardError

from optparse import OptionParser

import bs4
import requests
import requests_cache
import subprocess

from instances.models import Instance
from speeches.models import Section, Speaker, Speech


def prevnext(it):
    prev, curr, next = itertools.tee(it, 3)
    prev = itertools.chain([None], prev)
    next = itertools.chain(itertools.islice(next, 1, None), [None])
    return itertools.izip(prev, curr, next)


class ScrapingError(StandardError):
    """Exception to represent all different sorts of scraping error."""


class BaseParser(object):
    name_fixes = {}

    # Subclasses should override instance to be the label of an appropriate
    # Instance object.
    instance = None

    def __init__(self, cache_dir=None):
        if not self.instance:
            raise StandardError(
                "Set the 'instance' class attribute to an appropriate label.")

        if cache_dir is None:
            raise StandardError(
                'Set cache_dir to the directory for the requests cache.')

        self.parser = OptionParser()
        self._add_parser_options()
        (self.options, self.args) = self.parser.parse_args()
        self._process_parser_options()

        self.instance = self.get_or_create(Instance, label=self.instance)
        self.cache_dir = os.path.join(cache_dir, self.instance.label)

        try:
            os.makedirs(self.cache_dir)
        except:
            pass

        self.requests = requests_cache.core.CachedSession(cache_dir)

    def _add_parser_options(self):
        self.parser.add_option(
            '--commit',
            dest='commit',
            help='commit to database',
            action='store_true',
            )
        self.parser.add_option(
            '--process-existing',
            dest='process_existing',
            help="Process files already downloaded (default is to skip)",
            action='store_true',
            )

    def _process_parser_options(self):
        self.commit = self.options.commit
        self.process_existing = self.options.process_existing

    def get_transcripts(self):
        """Returns an iterator of dictionaries representing single transcripts.

        Each dictionary should be in the form expected by parse_transcript.

        OVERRIDE IN SUBCLASS.
        """
        raise NotImplementedError

    def get_pdf(self, pdf_url, name=None):
        if not name:
            name = os.path.basename(pdf_url)

        file_pdf = os.path.join(self.cache_dir, name)
        file_text = file_pdf.replace('.pdf', '.txt')

        if os.path.exists(file_text):
            if not self.process_existing:
                return
        else:
            with self.requests.cache_disabled():
                try:
                    pdf_transcript = self.get_url(pdf_url, 'binary')
                except (requests.exceptions.HTTPError, socket.error):
                    raise ScrapingError('Error fetching {}'.format(pdf_url))

            with open(file_pdf, 'w') as fp:
                fp.write(pdf_transcript)

            subprocess.call(['pdftotext', '-layout', file_pdf])

        text = open(file_text).read()

        # Be sure to have ^L on its own line
        text = text.replace('\014', '\014\n')
        # Return an array of lines
        return re.split('\r?\n', text)

    def get_url(self, url, type='none'):
        resp = self.requests.get(url)
        resp.raise_for_status()
        if type == 'binary':
            return resp.content
        elif type == 'html':
            return bs4.BeautifulSoup(resp.text)
        return resp.text

    def run(self):
        for data in self.get_transcripts():
            self.parse(data)

    def skip_transcript(self, data):
        return False

    def top_section_heading(self, data):
        return 'Hearing, %s' % data['date'].strftime('%d %B %Y').lstrip('0')

    def get_parent_section(self, data):
        """Find the section to create the top section in.

        All speeches for this transcript will be created in a section with
        heading provided by top_section_heading.

        Override this method to put this section inside another section.
        """
        return None

    def parse_transcript(self, data):
        """Takes transcript and returns an iterator of Speech objects.

        Find all the speeches in the transcript represented by the dictionary
        data and return an iterator of them as Speech objects.

        OVERRIDE IN SUBCLASS.
        """
        raise NotImplementedError

    def parse(self, data):
        if self.skip_transcript(data):
            return

        date = data.get('date')
        top_section = self.get_or_create(
            Section, instance=self.instance, source_url=data['url'],
            heading=self.top_section_heading(data),
            parent=self.get_parent_section(data),
        )

        for speech in self.parse_transcript(data):
            if not speech:
                continue

            if speech.section:
                if speech.section.object:
                    section = speech.section.object
                else:
                    heading = self.prettify(speech.section.heading)
                    section = Section(
                        instance=self.instance,
                        heading=heading,
                        parent=top_section,
                        )
                    if self.commit:
                        section.save()
                    speech.section.object = section
            else:
                section = top_section
            if speech.speaker:
                speaker = self.prettify(speech.speaker)
                speaker = self.get_or_create(
                    Speaker, instance=self.instance, name=speaker)
            else:
                speaker = None

            if not speech.type:
                speech.type = ('speech'
                               if speaker or speech.speaker_display
                               else 'narrative')

            text = '</p>\n<p>'.join([' '.join(s) for s in speech.text])
            text = '<p>%s</p>' % text
            speech_date = speech.date or date
            speech = Speech(
                instance=self.instance,
                section=section,
                text=text,
                speaker=speaker,
                speaker_display=speech.speaker_display,
                type=speech.type,
                start_date=speech_date,
                start_time=speech.time,
            )
            if self.commit:
                speech.save()

    def fix_name(self, name):
        name = name.title().replace('.', '')
        name = re.sub(
            'Mc[a-z]',
            lambda mo: mo.group(0)[:-1] + mo.group(0)[-1].upper(),
            name)
        name = self.name_fixes.get(name, name)
        return name

    def prettify(self, s):
        return s

    # Special get_or_create that won't always commit
    def get_or_create(self, model, **attrs):
        try:
            obj = model.objects.get(**attrs)
        except model.DoesNotExist:
            obj = model(**attrs)
            if self.commit:
                obj.save()
        return obj


class ParserSection(object):
    object = None

    def __init__(self, heading):
        self.heading = heading


class ParserSpeech(object):
    # Some state variables
    current_date = None
    current_time = None
    current_section = None
    witness = None

    def __init__(self, speaker, text, speaker_display=None, typ=None):
        self.speaker = speaker
        self.speaker_display = speaker_display
        self.text = [[text]]
        self.type = typ
        self.date = self.current_date
        self.time = self.current_time
        self.section = self.current_section

    def add_para(self, text):
        self.text.append([text])

    def add_text(self, text):
        self.text[-1].append(text)

    @classmethod
    def reset(cls, morning):
        cls.current_date = None
        cls.current_time = None
        if morning:
            cls.current_section = None
            cls.witness = None



from itertools import groupby

from django.forms.models import ModelChoiceIterator, ModelChoiceField

"""Common utility functions/classes
Things that are needed by multiple bits of code but are specific enough to
this project not to be in a separate python package"""

# From http://djangosnippets.org/snippets/2622/
class GroupedModelChoiceField(ModelChoiceField):
    def __init__(self, queryset, group_by_field, group_label=None, *args, **kwargs):
        """
        group_by_field is the name of a field on the model
        group_label is a function to return a label for each choice group
        """
        super(GroupedModelChoiceField, self).__init__(queryset, *args, **kwargs)
        self.group_by_field = group_by_field
        if group_label is None:
            self.group_label = lambda group: group
        else:
            self.group_label = group_label

    def _get_choices(self):
        """
        Exactly as per ModelChoiceField except returns new iterator class
        """
        if hasattr(self, '_choices'):
            return self._choices
        return GroupedModelChoiceIterator(self)
    choices = property(_get_choices, ModelChoiceField._set_choices)

# From http://djangosnippets.org/snippets/2622/
class GroupedModelChoiceIterator(ModelChoiceIterator):
    def __iter__(self):
        if self.field.empty_label is not None:
            yield (u"", self.field.empty_label)
        if self.field.cache_choices:
            if self.field.choice_cache is None:
                self.field.choice_cache = [
                    (self.field.group_label(group), [self.choice(ch) for ch in choices])
                        for group,choices in groupby(self.queryset.all(),
                            key=lambda row: getattr(row, self.field.group_by_field))
                ]
            for choice in self.field.choice_cache:
                yield choice
        else:
            for group, choices in groupby(self.queryset.all(),
                    key=lambda row: getattr(row, self.field.group_by_field)):
                yield (self.field.group_label(group), [self.choice(ch) for ch in choices])



import logging
import os
import tempfile
import shutil
from datetime import datetime, timedelta
import pytz

from django.test.utils import override_settings
from django.conf import settings

import speeches
from speeches.models import Speech, Recording, Section
from speeches.tests import InstanceTestCase

logging.disable(logging.WARNING)

@override_settings(MEDIA_ROOT=tempfile.mkdtemp())
class RecordingTests(InstanceTestCase):

    @classmethod
    def setUpClass(cls):
        cls._in_fixtures = os.path.join(os.path.abspath(speeches.__path__[0]), 'fixtures', 'test_inputs')

    def tearDown(self):
        # Clear the recordings folder if it exists
        recordings_folder = os.path.join(settings.MEDIA_ROOT, 'recordings')
        if(os.path.exists(recordings_folder)):
            shutil.rmtree(recordings_folder)

    def test_edit_recording_timestamps(self, filename='lamb.mp3'):
        # For now, just call the API to create the initial thing.
        # Probably should have some better way of doing this self-contained...
        SPEECHES = 4

        timestamps = '['
        for i in range(SPEECHES):
            t = 946684800 + i
            timestamps += '{"timestamp":' + str(t) + '000}'
            if i<SPEECHES-1:
                timestamps += ','
        timestamps += ']'

        audio = open(os.path.join(self._in_fixtures, filename), 'rb')
        resp = self.client.post('/api/v0.1/recording/', {
            'audio': audio,
            'timestamps': timestamps
        })

        recording = Recording.objects.order_by('-id')[0]
        self.assertIsNotNone(recording.audio)
        self.assertEqual(recording.audio_duration, 5)
        expected_timestamp = datetime.utcfromtimestamp(946684800).replace(tzinfo=pytz.utc)
        self.assertEqual(recording.start_datetime, expected_timestamp)
        self.assertEqual(recording.timestamps.count(), SPEECHES)
        self.assertEqual(Speech.objects.count(), SPEECHES)

        # Test each speech is a second apart
        last_start = None
        for s in Speech.objects.all():
            start = datetime.combine(s.start_date, s.start_time)
            if last_start:
                self.assertEqual(start - last_start, timedelta(seconds=1))
            last_start = start

        # Test assignment of section
        section = Section.objects.create(heading='A Section', instance=self.instance)
        self.assertEqual(Speech.objects.filter(section=section).count(), 0)
        resp = self.client.get('/recording/%s' % recording.id)
        self.assertContains(resp, 'A Section')
        resp = self.client.post('/recording/%s' % recording.id, { 'section': section.id })
        self.assertEqual(Speech.objects.filter(section=section).count(), SPEECHES)

        # XXX Perhaps django-webtest (which has form.submit()) or 
        # get things out of the view and test them independently.
        args = {
            "timestamps-TOTAL_FORMS": 4,
            "timestamps-INITIAL_FORMS": 4,
        }
        new = [ 0, 1, 3, 4 ]
        for i in range(SPEECHES):
            args['timestamps-%d-id' % i] = recording.timestamps.all()[i].id
            args['timestamps-%d-recording' % i] = recording.id
            args['timestamps-%d-timestamp' % i] = new[i]
        resp = self.client.post('/recording/%s/edit' % recording.id, args)

        # Now test the speeches are differently spaced apart
        last_start = None
        diffs = [ None, 1, 2, 1 ]
        for i, s in enumerate(Speech.objects.all()):
            start = datetime.combine(s.start_date, s.start_time)
            if last_start:
                self.assertEqual(start - last_start, timedelta(seconds=diffs[i]))
            last_start = start
